/*
    Kernel mode web server for Windows.
    Copyright (C) 2006-2009 Bo Brantn.
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <ntddk.h>
#include <ntstrsafe.h>
#include "ksocket.h"

#define INET_POOL_TAG 'aotn'

u_long __cdecl inet_addr(const char *name)
{
    struct in_addr addr;

    if (inet_aton(name, &addr))
    {
        return addr.s_addr;
    }
    return INADDR_NONE;
}

int __cdecl inet_aton(const char *name, struct in_addr *addr)
{
    u_int dots, digits;
    u_long byte;

    if(!name || !addr)
    {
        return 0;
    }

    for (dots = 0, digits = 0, byte = 0, addr->s_addr = 0; *name; name++)
    {
        if (*name == '.')
        {
            addr->s_addr += byte << (8 * dots);
            if (++dots > 3 || digits == 0)
            {
                return 0;
            }
            digits = 0;
            byte = 0;
        }
        else
        {
            byte = byte * 10 + (*name - '0');
            if (++digits > 3 || *name < '0' || *name > '9' || byte > 255)
            {
                return 0;
            }
        }
    }

    if (dots != 3 || digits == 0)
    {
        return 0;
    }

    addr->s_addr += byte << (8 * dots);

    return 1;
}

// standard but not reentrant.
char * __cdecl inet_ntoa(struct in_addr addr)
{
    static char buf[16];
    unsigned char *ucp = (unsigned char *)&addr;
    RtlStringCbPrintfA(buf, 16, "%d.%d.%d.%d", ucp[0] & 0xff, ucp[1] & 0xff, ucp[2] & 0xff, ucp[3] & 0xff);
    return buf;
}

// reentrant version.
char * __cdecl inet_ntoa_r(struct in_addr addr, char *buf)
{
    unsigned char *ucp = (unsigned char *)&addr;
    RtlStringCbPrintfA(buf, 16, "%d.%d.%d.%d", ucp[0] & 0xff, ucp[1] & 0xff, ucp[2] & 0xff, ucp[3] & 0xff);
    return buf;
}
